#include "city.h"
#include "control.h"
#include "glenable.h"

City::City(int num, vec2f pos, Explosions * expld)
{
	cityNum = num;
	position = pos;
	init(expld);
}

City::City(int num, float x, float y, Explosions * expld)
{
	cityNum = num;
	position.x = x;
	position.y = y;
	init(expld);
}

City::~City()
{
	// delete rockets;
}

void City::init(Explosions * expld)
{
	radius = RADIUS;
	maxRadius = RADIUS;
	minRadius = MIN_RADIUS;
	missilesFiredAt = 0;
	lastPainTime = -4;
	health = INIT_HEALTH;
	lastFiredAtTime = -4;
	getColour(cityNum);
	spd = 4;
	rockets = new Rockets(r,g,b,spd,expld);
	aimedRockets = new Rockets(0.7,0,0,2,expld);
	destroyed = false;
	rotation = 0;
	isCiv = false;
}

void City::Draw() const
{
	rockets -> drawPoints();
	aimedRockets -> drawPoints();
	if(destroyed)
		return;
	glPushMatrix();
	glPushAttrib(GL_CURRENT_BIT);
	glTranslatef(position.x,position.y,0);
	glRotatef(-rotation,0,0,1);
	int i = 0;
	for(set<int>::iterator it = cityPlayers.begin(); it != cityPlayers.end(); it ++) {
		applyColour(*it);
		
		glLineWidth(2);
		//drawHalfCircle(radius,32,position.x,position.y);
		drawHalfCircle(radius - (float) (radius * 2 * (float) i / 10.0),32);
		//cout << radius - (float) (radius * (float) i / 10.0) << " " << radius << endl;
		i ++;
	}
	if(!i) {
		glColor3f(r,g,b);
		glLineWidth(3);
		//drawHalfCircle(radius,32,position.x,position.y);
		drawHalfCircle(radius,32);
	}
	
	glLineWidth(1);
	glPopAttrib();
	glPopMatrix();
}

void City::update(float dt)
{
	rockets -> updatePoints(dt);
	aimedRockets -> updatePoints(dt);
}

int City::GetNumPlayers() const
{
	return (int) cityPlayers.size();
}

set<int> City::GetPlayers() const
{
	return cityPlayers;
}

bool City::SetPlayer(const int ref)
{
	cityPlayers.insert(ref);
	return true;
}

void City::DropPlayer(const int ref)
{
	cityPlayers.erase(ref);
}

float City::Radius() const
{
	return radius;
}

void City::SetRadius(const float rad)
{
	radius = radius / maxRadius * rad;
	maxRadius = rad;
	minRadius = rad / 2.0;
}

void City::SetRotation(const float rot)
{
	rotation = rot;
}

void City::SetPosition(const vec2f pos)
{
	position = pos;
}

float City::Rotation() const
{
	return rotation;
}

void City::FireMissile(const vec2f playerAim, const float playerPower)
{
	if(!destroyed) {
		vec2f missileAim;
		missileAim = playerAim * radius * 1.5;
		missileAim.normalize();
		missileAim *= playerPower;
		rockets -> createPoint(position + (playerAim  * radius * 1.5), missileAim + position);
	}
}

bool City::IsControlled() const
{
	return (cityPlayers.size() != 0);
}

bool City::IsActive() const
{
	return (IsControlled() || rockets -> points.size() > 0 || aimedRockets -> points.size() > 0);
}

bool City::ReadyToBeFiredAt(bool aimedAt)
{
	float timer = TIME_LAPSE - (float) dead/20.0;
	int missiles = MAX_OFF_MISSILES + dead / 20;
	if(aimedAt) {
		timer *= 0.5;
		missiles *= 2;
	}
	if(timer >= MIN_TIME_LAPSE)
		timer = MIN_TIME_LAPSE;
	if(missiles >= cMax)
		missiles = cMax;
		
	return (!destroyed && IsControlled() && (clock() - lastFiredAtTime) * 0.001 >= timer && aimedRockets -> size() < missiles);
}

vec2f City::Pos() const
{
	return position;
}

void City::getColour(const int switchNum)
{
	switch(switchNum) {
		case 0:		// 0
			r = 0.2;
			g = 1;
			b = 0.2;
			break;
		case 1:		// 1
			r = 0;
			g = 1;
			b = 1;
			break;
		case 2:		// 2
			r = 0;
			g = 1;
			b = 0;
			break;
		case 3:		// case 3
			r = 0;
			g = 0;
			b = 1;
			break;
		case 4:		// case 4
			r = 1;
			g = 1;
			b = 0;
			break;
		case 5:		// 5
			r = 1;
			g = 0;
			b = 1;
			break;
		case 6:		// 6
			r = 0.5;
			g = 0.5;
			b = 1;
			break;
		case 7:		// 7
			r = 0.2;
			g = 0.5;
			b = 0.8;
			break;
		default:
			r = 1;
			g = 1;
			b = 1;
			break;
	}
}

void City::MissileFiredAt()
{
	float xDiff = (Max.x - Min.x);
	//missilesFiredAt ++;
	lastFiredAtTime = clock();
	vec2f start;
	start.x = RAND_UNIT*xDiff-xDiff/2;
	start.y = Max.y;
	vec2f dev;
	dev.x = (RAND_UNIT-0.5)*RADIUS*2;
	dev.y = -0.1;
	aimedRockets -> createPoint(start, position + dev);
}

void City::MissileDestroy(int i, bool enemy)
{
	if(enemy) {
		missilesFiredAt --;
		aimedRockets -> deletePoint(i);
	} else {
		rockets -> deletePoint(i);
	}
}

bool City::TakeDamage(int amount)
{
	if(!IsControlled() || (clock() - lastPainTime) * 0.001 < LIFETIME * 1.05)
		return false;
	lastPainTime = clock();
	health -= amount;
	radius = minRadius + ((maxRadius - minRadius) * health / INIT_HEALTH);
	return health <= 0;
}

void City::Destroyed()
{
	destroyed = true;
	cityPlayers.clear();
}

bool City::IsAlive() const
{
	return !destroyed;
}

bool City::IsCiv() const
{
	return isCiv;
}

void City::SetY(const float yVal)
{
	float yDiff = Max.y - Min.y;
	position.y = yVal;
	aimedRockets -> setSpeed(yDiff * ROCKET_RATE);
	rockets -> setSpeed(yDiff * 2 * ROCKET_RATE);
}






















void applyColour(int pn)
{
	switch(pn) {
		case 0:
			glColor3f(0,0,1);
			break;
		case 1:
			glColor3f(1,1,0);
			break;
		case 2:
			glColor3f(0,1,0);
			break;
		case 3:
			glColor3f(1,0,1);
			break;
		case 4:
			glColor3f(0,1,1);
			break;
		case 5:
			glColor3f(0.5,0.5,1);
			break;
		case 6:
			glColor3f(0.2,1,0.2);
			break;
		case 7:
			glColor3f(0.2,0.5,0.8);
			break;
		default:
			glColor3f(1,1,1);
			break;
	}
}

string getColourString(int pn)
{
	switch(pn) {
		case 0:
			return "Blue";
		case 1:
			return "Yellow";
		case 2:
			return "Green";
		case 3:
			return "Purple";
		case 4:
			return "Teal";
		case 5:
			return "Grey";
		case 6:
			return "Light Green";
		case 7:
			return "Light Blue";
	}
	return "";
}