#include "control.h"
#include "player.h"
#include "vec.h"
#include "city.h"
#include "display.h"
#include "cityEarth.h"
#include "cityCiv.h"
#include <set>

//#define FRAME_RATE
#define CITY_CHANGE 8
float timer = 0;
float gameOverTime = 0;
int STATE = PRE_GAME;
int cityAim = 0;
int cityTime = 0;
int keyCheckTime = KEYBOARD_CHECK_TIME;
bool drawAllCities = true;
bool restrictAiming = true;
bool quitNoPlayers = false;
bool earthEnd = false;
int score = 0;
int dead = 0;
const static int earthSectors = 128;
const static int groundSectors = 200;

Explosions explosions;
ParticleContainer pinger(1,1,1,1);
Rockets missiles(0.7, 0.0, 0.0, 2.0,&explosions);
vec2f groundCoords[1000];

#ifdef FRAME_RATE
	int frameCounter = 0;
	float frameRateTimer = 0;
#endif

vector<City *> cities;		// cities
vector<Player> players;		// players who have entered input of some kind
int currPlayers = 0;
vector<int> addPlayers;		// players who want to take control of a city
vector<int> dropPlayers;	// players who want to drop out of game

Earth * earth;

vec2f Min = vec2f(SCREEN_MIN_X,SCREEN_MIN_Y);		// for glOrtho()
vec2f Max = vec2f(SCREEN_MAX_X,SCREEN_MAX_Y);

float yBase;							// for spread of cities
float stepValue = RADIUS * 6;
float firstCityPos = Min.x + stepValue * 2.0 / 3.0-0.02;

int numCities = MAX_CITIES;

void init()
{
	float displacement[101];
	int i = 0;
	int j = 0;
	float k = 0.0;
	int l = 0;
	srand(time(NULL));
	for(i = 0; i < 1000; i++){
		groundCoords[i].x = 0;
		groundCoords[i].y = 0;
	}

	for(i = 0; i <= 100; i++){
		displacement[i] = RAND_UNIT-0.5;
	}
	for(j = 0; j < 2; j ++){
		if(gameMode == EARTH)
			displacement[0] = (displacement[1]+displacement[0]+displacement[100])/3;
		for(i = 1; i < 100; i++){
			displacement[i] = (displacement[i-1]+displacement[i]+displacement[i+1])/3;
		}
		if(gameMode == EARTH)
			displacement[100] = (displacement[99]+displacement[0]+displacement[100])/3;
	}
	switch(gameMode) {
		City * temp;
		int arrayPos;
		case SINGLE:
			numCities = 7;
			stepValue = RADIUS * 6.75;
			firstCityPos = Min.x + stepValue * 2.0 / 3.0;
			setYBase();
			l = 0;
			for(i = 0; i <= groundSectors; i++){
				groundCoords[i].x = (float)i/(groundSectors/(-Min.x + Max.x))-1;
				groundCoords[i].y = yBase + displacement[l]/20;
				l++;
				if(l > 100)
					l = 0;
			}
			for(i = 0; i < numCities; i ++) {
				if(i == 3){
					temp = new City(i,firstCityPos + i*stepValue,yBase,&explosions);
					// temp->SetRadius(0.15);
				}
				else
					temp = new CityCiv(i,firstCityPos + i*stepValue,yBase,&explosions);
				cities.push_back(temp);
				arrayPos = getPositionPoint(temp->Pos());
				flattenGround(arrayPos);
			}
			break;
		case HALF:
			numCities = MAX_CITIES * 0.5;
			stepValue = RADIUS * 6 * 2;
			firstCityPos = Min.x + stepValue * 2.0 / 3.5;
		case STD:
			setYBase();
			l = 0;
			for(i = 0; i <= groundSectors; i++){
				groundCoords[i].x = (float)i/(groundSectors/(-Min.x + Max.x))-1;
				groundCoords[i].y = yBase + displacement[l]/20;
				l++;
				if(l > 100)
					l = 0;
			}
			for(i = 0; i < numCities; i ++) {
				temp = new City(i,firstCityPos + i*stepValue,yBase,&explosions);
				cities.push_back(temp);
				arrayPos = getPositionPoint(temp->Pos());
				flattenGround(arrayPos);
			}
			break;
		case EARTH:
			Min = vec2f(SCREEN_MIN_X * 1.25,SCREEN_MIN_Y * 1.25);
			Max = vec2f(SCREEN_MAX_X * 1.25,SCREEN_MAX_Y * 1.25);
			drawAllCities = false;
			earthEnd = false;
			yBase = Min.y * 2;
			i = j = 0;
			for(k = 0; k < 2*PI-0.001; k += PI/(earthSectors/2), i++){
				groundCoords[i].x = cos(k)/2 + cos(k)*displacement[j]/20;
				groundCoords[i].y = sin(k)/2 + sin(k)*displacement[j]/20;
				j++;
				if(j > 100){
					j = 0;
				}
			}
			earth = new Earth(groundCoords,earthSectors,&explosions);
			break;
	}
	
	ReSizeGLScene(nWidth,nHeight);
#ifdef HIGH_QUAL
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);
#endif
}

int initKeyboard(int kbn)
{
	int p = searchForPlayer(kbn);
	if(p != -1)
		return p;
	Player temp(kbn, players.size());
	players.push_back(temp);
	addPlayers.push_back(players.size() - 1);
	return (int) players.size() - 1;
}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	switch(STATE) {
		case PRE_GAME:
			drawPreGame();
			break;
		case IN_GAME:
		case GAME_OVER:
			switch(gameMode) {
	 			case SINGLE:
	 				drawPowerPoints(true);
	 			case STD:
	 				displayStandardGame();
	 				break;
	 			case HALF:
	 				displayHalfGame();
	 				break;
	 			case EARTH:
	 				drawEarthGame();
	 				break;
	 		}
			break;
		case POST_GAME:
			drawPostGame();
			break;
		
	}

	#ifdef FRAME_RATE
		drawFrameRate(fps);
	#endif
}

int getPositionPoint(vec2f posPoint){
	float point = posPoint.x;
	float maximum = -Min.x + Max.x;
	point += -Min.x;
	point /= maximum;
	point *= groundSectors;
	return point;
}

void flattenGround(int point){
	int i, k;
	int jumps = 6;//4 for 100, 6 for 200
	for(i = point-4; i < point + jumps; i++){//1 for 100, 3 for 200
		k = i;
		if(k > groundSectors || k < 0)
			continue;
		groundCoords[k].y = yBase;
	}
}

void deformGround(int point){
	int i, k;
	float j, l;
	static const float maxDev = -0.01;
	static const float devDrop = 0.005;
	j = maxDev;
	int jumps = 5;
	for(i = point - jumps, l = jumps/2; i < point; i++, l -= 0.5){
		k = i;
		if(k > groundSectors || k < 0)
			continue;
		groundCoords[k].y -= j;
		j += devDrop*l;
	}
	j = maxDev;
	for(i = point + jumps, l = jumps/2; i >= point; i--, l -= 0.5){
		k = i;
		if(k > groundSectors || k < 0)
			continue;
		groundCoords[k].y -= j;
		j += devDrop*l;
	}
	for(i = 0; i < (int) cities.size(); i ++){
		if(cities[i]->IsAlive()){
			int arrayPos = getPositionPoint(cities[i]->Pos());
			flattenGround(arrayPos);
		}
	}
}

void drawGround()
{
	glPushAttrib(GL_CURRENT_BIT);
	glColor3f(0.5,0.25,0);

#ifdef HIGH_QUAL
	glBegin(GL_LINE_LOOP);
	glVertex2f(0,Min.y-2);
	for(float i = 0; i <= groundSectors; i ++){
		glVertex2f(groundCoords[(int)i].x, groundCoords[(int)i].y);
	}
	glVertex2f(0,Min.y-2);
	glEnd();
#endif
	glBegin(GL_POLYGON);
	glVertex2f(0,Min.y-2);
	glVertex2f(Min.x - 0.1,Min.y-2);
	for(float i = 0; i <= groundSectors; i ++){
		// if(groundCoords[(int)i].y == yBase)
			// glColor3f(1,1,1);
		glVertex2f(groundCoords[(int)i].x, groundCoords[(int)i].y);
		// if(groundCoords[(int)i].y == yBase)
			// glColor3f(0.5,0.25,0);
	}
	glVertex2f(Max.x + 0.1, Min.y-2);
	glEnd();
	glPopAttrib();
}

int getRotationPoint(vec2f rotPoint){
	vec2f begin;
	begin.x = 1;
	begin.y = 0;
	float angle = acos(((rotPoint.x * begin.x) + (rotPoint.y * begin.y)) / 
					(sqrt(rotPoint.x*rotPoint.x + rotPoint.y*rotPoint.y) * 
					sqrt(begin.x*begin.x + begin.y*begin.y)));
	if(rotPoint.y < 0){
		angle = PI-angle+PI;
	}
	angle /= 2*PI;
	int point = floor(angle * earthSectors);
	return point;
}

void drawScore(const float xpos, const float ypos)
{
	char buf[64];
	//snprintf(buf,64,"Team Score: %d - Missiles Destroyed: %d",score,dead);
	sprintf(buf, "Team Score: %d - Missiles Destroyed: %d",score,dead);
	renderText(buf,xpos,ypos);
	//sprintf(buf,"%d players",currPlayers);
	//renderText(buf,300,300);
}

void update(float dt)
{
	timer += dt;
	#ifdef FRAME_RATE
		frameCounter ++;
		frameRateTimer += dt;
		if(frameRateTimer >= FRAME_TIME) {
			fps = frameCounter / (frameRateTimer);
			frameRateTimer = 0;
			frameCounter = 0;
		}
	#endif

	if(STATE == POST_GAME)
		return;

	if(STATE == GAME_OVER) {
		if((clock() - gameOverTime) * 0.001 > POST_GAME_DELAY) {
			STATE = POST_GAME;
		}
		return;
	}

	if(STATE == PRE_GAME) {
		if(timer - dt < PRE_TIME && timer >= PRE_TIME) {
			STATE = IN_GAME;
			init();
		}
		return;
	}

	if(timer >= keyCheckTime) {
		checkForDroppedPlayers();
		keyCheckTime += KEYBOARD_CHECK_TIME;
	}

	missiles.updatePoints(dt);
	explosions.updatePoints(dt);
	pinger.updatePoints(dt);

	for(int i = 0; i < (int) dropPlayers.size(); i ++) {
		int player = dropPlayers[i];
		int city = players[player].GetCity();

		if(city != NO_CITY) {
			cities[city]->DropPlayer(player);
		}
		players[player].SetCity(NO_CITY,vec2f(0,0),0);
		currPlayers --;
	}
	dropPlayers.clear();

	for(int i = 0; i < (int) addPlayers.size(); i ++) {
		int city;
		int player = addPlayers[i];
		int currCity = players[player].GetCity();
		switch(gameMode) {
			case SINGLE:
				players[player].SetAimLimits(0,AIM_DIV / 2);
				city = cities.size() / 2;
				if(cities[city]->SetPlayer(player)) {
					players[player].SetCity(city,cities[city] -> Pos(), cities[city] -> Radius());
					players[player].SetVAimCentre(cities[city] -> Pos());
					currPlayers ++;
				}
				 break;
			case STD:
			case HALF:
				players[player].SetAimLimits(0,AIM_DIV / 2);
				if(currCity != NO_CITY) {
					currPlayers --;
					cities[currCity]->DropPlayer(player);
					city = currCity + 1;
					if(city == (int) cities.size())
						city = 0;
					for(int j = 0; j < (int)cities.size() && !cities[city]->IsAlive(); j ++) {
						city ++;
						if(city == (int) cities.size())
							city = 0;
					}
				} else {
					city = getNextAvailCity();
				}

				if(city == -1) 
					break;
				currPlayers ++;

				if(cities[city]->SetPlayer(player)) {
					players[player].SetCity(city,cities[city] -> Pos(), cities[city] -> Radius());
					players[player].SetVAimCentre(cities[city] -> Pos());
					if(restrictAiming && cities[city] -> GetNumPlayers() > 1)
						setPlayerAimLimits(city);
				}
				break;
			case EARTH:
				float aimPoint;
				city = getUncontrolledCity();
				if(city == -1) {
					vec2f loc;
					while(true) {
						bool move = true;
						float random = RAND_UNIT * 256;

						float theta = 2.0 * random * pi / 256;
						loc.x = 0.5 * cos(theta);
						loc.y = 0.5 * sin(theta);
						for(int j = 0; j < (int) cities.size(); j ++) {
							vec2f temp = cities[j] -> Pos();
							if(ABS(getRotationPoint(temp) - getRotationPoint(loc)) <= 3) {
								move = false;
								break;
							}
						}
						if(move)
							break;
					}
					CityEarth * temp = new CityEarth(cities.size(),loc,&explosions);
					cities.push_back(temp);
					city = (int) cities.size() - 1;
				}
				if(cities[city]->SetPlayer(player))
					players[player].SetCity(city,cities[city] -> Pos(), cities[city] -> Radius());
				aimPoint = cities[city]->Rotation();
				players[player].SetAimPoint((90.0 - aimPoint) * AIM_DIV / 360.0);
				currPlayers ++;
				break;
		}
	}
	addPlayers.clear();

	if(gameMode == EARTH) {
		earth -> update(dt);
		if(earth -> ReadyToBeFiredAt(false))
			earth -> MissileFiredAt();
		
		for(int rockets = (int) earth -> aimedRockets -> size() - 1; rockets >= 0; rockets --) {
			int arrayPos = getRotationPoint(earth->aimedRockets->points[rockets].Pos);
			if(earth->aimedRockets->points[rockets].Pos.sizesq() <= groundCoords[arrayPos].sizesq()) {
				earth -> DeformEarth(arrayPos);
				earth->MissileDestroy(rockets,true);
				levelCities();
			}
			for(int explosion = (int)explosions.points.size() - 1; explosion >= 0; explosion --) {
				vec2f distance = earth->aimedRockets -> points[rockets].Pos - explosions.points[explosion].Pos;
				if(distance.sizesq() <= explosions.points[explosion].speed * explosions.points[explosion].speed) {
					earth->MissileDestroy(rockets,true);
					score += SCORE_INCREASE;
					dead ++;
				}
			}
		}
	}

	for(int i = 0; i < (int) cities.size(); i ++) {
		cities[i]->update(dt);

		if(!cities[i]->IsActive())
			continue;

		set<int> cityPlayers = cities[i]->GetPlayers();

		for(int explosion = (int)explosions.points.size() - 1; explosion >= 0; explosion --) {
			Particle e = explosions.points[explosion];
			Particle r;
			vec2f distance = e.Pos - cities[i]->Pos();
			if(gameMode != EARTH && distance.sizesq() <= (cities[i]->Radius() + e.speed) * (cities[i]->Radius() + e.speed)) {
				if(cities[i]->IsControlled() && cities[i]->TakeDamage(1)) {
					cities[i]->Destroyed();
					for(set<int>::iterator it = cityPlayers.begin(); it != cityPlayers.end(); it ++) {
						players[*it].SetCity(NO_CITY,vec2f(0,0),0);
						addPlayers.push_back(*it);
						currPlayers --;
					}
				}
			}

			for(int rockets = (int)cities[i]->aimedRockets -> points.size() - 1; rockets >= 0; rockets --) {
				r = cities[i]->aimedRockets -> points[rockets];
				distance = r.Pos - e.Pos;
				if(distance.sizesq() <= e.speed * e.speed) {
					cities[i]->MissileDestroy(rockets,true);
					score += SCORE_INCREASE;
					dead ++;
				}
			}
		}
		
		switch(gameMode) {
			case SINGLE:
			case STD:
			case HALF:
				for(set<int>::iterator it = cityPlayers.begin(); it != cityPlayers.end(); it ++) {
					if(players[*it].ClassicAim() && players[*it].keys[SHIFT])
						players[*it].update(dt * 2);
					else
						players[*it].update(dt);

					players[*it].CorrectVAim(false);
				}

				for(int rockets = (int)cities[i]->aimedRockets->points.size() - 1; rockets >= 0; rockets --) {
					int arrayPos = getPositionPoint(cities[i]->aimedRockets->points[rockets].Pos);
					if(cities[i]->aimedRockets->points[rockets].Pos.y <= groundCoords[arrayPos].y) {
						deformGround(arrayPos);
						cities[i]->MissileDestroy(rockets,true);
					}
				}
				if(gameMode == SINGLE) {
					if(cities[i] -> IsCiv() && cities[i]->ReadyToBeFiredAt(false)) {
						cities[i]->MissileFiredAt();
					}
					break;
				}

				if(cities[i]->ReadyToBeFiredAt(i == cityAim)) {
					cities[i]->MissileFiredAt();
				}

				if(currPlayers != 0 && timer >= cityTime) {
					cityTime += CITY_CHANGE;
					getNextCity();
				}

				break;
			case EARTH:
				set<int>::iterator it = cityPlayers.begin();
				float oldAim = players[*it].GetAimPoint();
				float oldPower = players[*it].Power();
				if(players[*it].ClassicAim() && players[*it].keys[SHIFT])
					players[*it].update(dt * 2);
				else
					players[*it].update(dt);
				float newAim = players[*it].GetAimPoint();
				if(newAim != oldAim) {
					players[*it].CorrectVAim(true);
					vec2f aim = players[*it].GetAim();
					int arrayPos = getRotationPoint(aim);
					bool move = true;
					for(int j = 0; j < (int) cities.size(); j ++) {
						if(j == i)
							continue;
						vec2f temp = cities[j] -> Pos();
						if(ABS(getRotationPoint(temp) - arrayPos) <= 3) {
							players[*it].SetAimPoint(oldAim);
							if(players[*it].ClassicAim())
								players[*it].SetPower(oldPower);
							players[*it].ResetVAim();
							players[*it].CorrectVAim(true);
							move = false;
							break;
						}
					}
					if(move) {
						float dev = groundCoords[arrayPos].size();
						cities[i]->SetPosition(aim * dev);
						players[*it].UpdateCityPos(aim * dev);
					}
				}
				break;
		}
	}
	if(checkEndGame()) {
		STATE = GAME_OVER;
		gameOverTime = clock();
	}
}

void keyUp(unsigned short key, int keyboardRef)
{
	int player = searchForPlayer(keyboardRef);
	if(player == -1 || players[player].GetCity() == NO_CITY)
		return;
	players[player].keys[key] = false;
	switch(key) {
		case SPACE_BAR:
			if(players[player].CanFire()) {
				cities[players[player].GetCity()]->FireMissile(players[player].GetAim(),players[player].Power());
				players[player].Fired();
			}
			if(!players[player].ClassicAim())
				players[player].ResetPower();
			break;
	}
}

void keyDown(unsigned short key, int keyboardRef)
{
	if(STATE == POST_GAME) {
		resetGame();
		return;
	}
	if(STATE == PRE_GAME) {
		initKeyboard(keyboardRef);
		return;
	}

	int player = searchForPlayer(keyboardRef);
	if((player == -1 || players[player].GetCity() == NO_CITY) && key != ENTER)
		return;

	switch(key) {
		case LEFT_ARROW:
		case RIGHT_ARROW:
		case SPACE_BAR:
			players[player].keys[key] = true;
			break;
		case 'W':
		case 'A':
		case 'S':
		case 'D':
			if(players[player].ClassicAim())
				players[player].keys[key] = true;
			break;
		case ENTER:
			if(player == -1) {
				initKeyboard(keyboardRef);
			} else if(players[player].GetCity() == NO_CITY) {
				addPlayers.push_back(player);
			}
			break;
		case 'P':
			pinger.ring(cities[players[player].GetCity()]->Pos());
			break;
		case ESCAPE:
			dropPlayers.push_back(player);
			if(quitNoPlayers && currPlayers - dropPlayers.size() == 0)
				PostQuitMessage(0);
			break;
		case TAB:
			if(gameMode != SINGLE && gameMode != EARTH)
				addPlayers.push_back(player);
			break;
		case 'Q':
			PostQuitMessage(0);
			break;
		case 'C':
			players[player].ToggleClassicAim();
			break;
		case 'R':
			if(gameMode == EARTH)
				break;
			restrictAiming = !restrictAiming;
			if(restrictAiming) {
				for(int i = 0; i < (int) cities.size(); i ++)
					setPlayerAimLimits(i);
			} else {
				unRestrictAiming();
			}
			break;
		case 'I':
			displayInfo();
			break;
		case 'B':
			if(gameMode == EARTH)
				break;
			drawAllCities = !drawAllCities;
			break;
		case 'X':
			quitNoPlayers = !quitNoPlayers;
			break;
		default:
			players[player].keys[key] = true;
			break;
	}
}

int searchForPlayer(int kbn)
{
	for(int i = 0; i < (int) players.size(); i ++) {
		if(players[i].IsKeyboard(kbn)) {
			return i;
		}
	}
	return -1;
}

int getNextAvailCity()
{
	int num = ((int) cities.size() - 1) / 2;
	for(int i = 0; i < (int) cities.size(); i ++) {

		int quot = getNumAliveCities();
		if(!quot)
			return -1;
		int split = ((currPlayers + 1) / quot);
		int temp;
		if(i%2)
			temp = num + (i + 1) / 2;
		else
			temp = num - (i + 1) / 2;

		if(cities[temp]->IsAlive() && cities[temp]->GetNumPlayers() <= split)
			return temp;
	}

	return -1;
}

void checkForDroppedPlayers()
{
	vector<int> keyboards;
	if(!GetActiveKeyboards(keyboards))
		return;

	for(int i = 0; i < (int) players.size(); i ++) {
		bool attached = false;
		for(int j = 0; j < (int) keyboards.size(); j ++) {
			if(players[i].IsKeyboard(keyboards[j])) {
				attached = true;
			}
		}
		if(!attached)
			dropPlayers.push_back(i);
	}
}

int getNextCity()
{
	for(int i = cityAim + 1; ;i ++) {
		if(i == (int)cities.size())
			i = 0;
		if(cities[i]->IsControlled() || i == cityAim) {
			cityAim = i;
			break;
		}
	}
	return cityAim;
}

float setYBase()
{
	yBase = Min.y + (Max.y - Min.y) / 10.0;
	for(int i = 0; i < (int)cities.size(); i ++)
		cities[i]->SetY(yBase);
	return yBase;
}

int getNumAliveCities()
{
	int count = 0;
	for(int i = 0; i < (int) cities.size(); i ++) {
		if(cities[i]->IsAlive())
			count ++;
	}
	return count;
}

void resetGame() 
{
	STATE = PRE_GAME;
	timer = 0;
	players.clear();
	cities.clear();
	explosions.erase();
	pinger.erase();
	addPlayers.clear();
	dropPlayers.clear();
	currPlayers = 0;
	dead = 0;
	score = 0;
	earthEnd = false;
}

int getUncontrolledCity()
{
	for(int i = 0; i < (int) cities.size(); i ++) {
		if(!cities[i] -> IsControlled())
			return i;
	}
	return -1;
}

void setPlayerAimLimits(const int city)
{
	if(cities[city] -> IsCiv())
		return;
	set<int> cityPlayers = cities[city] -> GetPlayers();
	int numPlayers = (int) cityPlayers.size();
	int i = 0;
	float fieldOfFire = 180;
	float overlapRatio = 12;
	float fofnp = fieldOfFire / numPlayers;
	float fofor = fieldOfFire / overlapRatio;
	for(set<int>::iterator it = cityPlayers.begin(); it != cityPlayers.end(); it ++) {
		float minAim = (0 + i * (fofnp - fofor)) / fieldOfFire * AIM_DIV / 2;
		float maxAim = (fieldOfFire - (numPlayers - 1 - i) * (fofnp - fofor)) / fieldOfFire * AIM_DIV / 2;
		players[*it].SetAimLimits(minAim, maxAim);
		players[*it].SetAimPoint((maxAim + minAim) / 2);
		i ++;
	}

}

void unRestrictAiming()
{
	for(int i = 0; i < (int) players.size(); i ++) {
		players[i].SetAimLimits(0,AIM_DIV / 2);
	}
}

void levelCities()
{
	for(int i = 0; i < (int) cities.size(); i ++) {
		if(!cities[i] -> IsControlled())
			continue;
		set<int> cityPlayers = cities[i] -> GetPlayers();
		vec2f aim = players[*(cityPlayers.begin())].GetAim();
		int arrayPos = getRotationPoint(aim);
		float dev = groundCoords[arrayPos].size();
		cities[i]->SetPosition(aim * dev);
	}
}

int numCivCities()
{
	int count = 0;
	for(int i = 0; i < (int) cities.size(); i ++) {
		if(cities[i] -> IsCiv() && cities[i] -> IsAlive())
			count ++;
	}
	return count;
}

bool checkEndGame()
{
	switch(gameMode) {
		case STD:
		case HALF:
			return (getNumAliveCities() == 0);
		case SINGLE:
			return (numCivCities() == 0);
		case EARTH:
			return earthEnd;
	}
	return false;
}