#include "earth.h"
#include "glenable.h"
#include "control.h"

extern int currPlayers;
extern const float pi;
extern bool earthEnd;

Earth::Earth(vec2f *coords, int num, Explosions * expld) : City(0,vec2f(0,0),expld)
{
	groundCoords = coords;
	size = num;
	aimedRockets -> setSpeed(ROCKET_RATE);
}

void Earth::DeformEarth(int point)
{
	int i, k;
	float j, l;
	static const float maxDev = 1.03;
	static const float devDrop = 0.01;
	j = maxDev;
	int jumps = size/20;
	for(i = point - jumps, l = jumps/2; i < point; i++, l -= 0.5){
		k = i;
		if(k >= size)
			k -= size;
		if(k < 0)
			k += size;
		groundCoords[k] *= j;
		j -= devDrop*l;
	}
	j = maxDev;
	for(i = point + jumps, l = jumps/2; i >= point; i--, l -= 0.5){
		k = i;
		if(k >= size)
			k -= size;
		if(k < 0)
			k += size;
		groundCoords[k] *= j;
		j -= devDrop*l;
	}
}

void Earth::Draw() const
{
	aimedRockets -> drawPoints();
	glPushAttrib(GL_CURRENT_BIT);
	glColor3f(0.5,0.25,0);
	int j = 0;
	static const float deadSize = 0.35;

#ifdef HIGH_QUAL
	glBegin(GL_LINE_LOOP);
	for(j = 0; j < size; j++){
		glVertex2f(groundCoords[j].x, groundCoords[j].y);
	}
	glEnd();
#endif

	glBegin(GL_TRIANGLE_FAN);
	glVertex2f(0,0);
	for(j = 0; j < size; j++){
		glVertex2f(groundCoords[j].x, groundCoords[j].y);
		if(groundCoords[j].size() < deadSize){
			earthEnd = true;
		}
	}
	j = 0;
	glVertex2f(groundCoords[j].x, groundCoords[j].y);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
	glColor3f(0.5,0,0);
	glVertex2f(0,0);
	glColor3f(0.5,0.225,0);
	drawCircle(deadSize, size, 0, 0);
	glEnd();
	glPopAttrib();
}

bool Earth::IsControlled() const
{
	return true;
}

bool Earth::ReadyToBeFiredAt(bool aimedAt)
{
	float timer = TIME_LAPSE - (float) dead/40.0;
	int missiles = MAX_OFF_MISSILES * currPlayers + dead / 40;

	if(timer >= MIN_TIME_LAPSE)
		timer = MIN_TIME_LAPSE;
	if(missiles >= cMax)
		missiles = cMax;

	return (!destroyed && (clock() - lastFiredAtTime) * 0.001 >= timer && aimedRockets -> size() < missiles);
}

void Earth::MissileFiredAt()
{
	float random = RAND_UNIT * 256;
	vec2f start;
	lastFiredAtTime = clock();

	float theta = 2.0 * random * pi / 256;
	start.x = 2.25 * cos(theta);
	start.y = 2.25 * sin(theta);
	vec2f dev;
	dev.x = (RAND_UNIT-0.5)/2;
	dev.y = (RAND_UNIT-0.5)/2;
	aimedRockets -> createPoint(start, position + dev);
}