#include "player.h"
#include <string.h>
#include "glenable.h"

extern vec2f Min;
extern vec2f Max;

Player::Player(int ref, int num)
{
	keyboardRef = ref;
	playerNum = num;
	cityNum = NO_CITY;
	aimPoint = AIM_DIV / 4.0;
	power = POWER_MIN;
	lastFireTime = -4;
	missilesFired = 0;
	lowAim = AIM_DIV * 2;
	highAim = AIM_DIV * 2;
	classicAim = false;
	vAimCentre = vec2f(0,0);

	memset(keys,0,256);
	calcAim();
}

void Player::Draw() const
{
	glPushAttrib(GL_CURRENT_BIT);
		glLineWidth(3);
		applyColour(playerNum);
		vec2f drawAim = aim;
		drawAim.x *= cityRadius * 1.5;
		drawAim.y *= cityRadius * 1.5;
		drawAim += cityPos;
		glBegin(GL_LINES);
			glVertex2f(cityPos.x,cityPos.y);
			glVertex2f(drawAim.x,drawAim.y);
		glEnd();
		//if(classicAim)
		//	AimDraw();
		glLineWidth(1);
	glPopAttrib();
}

void Player::AimDraw() const
{
	glPushAttrib(GL_CURRENT_BIT);
		glLineWidth(3);
		applyColour(playerNum);
		glBegin(GL_LINES);
			float xfac = 0.03 * (Max.x - Min.x) / (Max.y - Min.y) / 2;
			if(classicAim) {
				glVertex2f(vAim.x-xfac,vAim.y);
				glVertex2f(vAim.x+xfac,vAim.y);
				glVertex2f(vAim.x,vAim.y-0.03);
				glVertex2f(vAim.x,vAim.y+0.03);
			} else {
				vec2f drawAim = aim;
				drawAim.x *= cityRadius * 1.5;
				drawAim.y *= cityRadius * 1.5;
				drawAim += cityPos;
				drawAim += aim * power;
				glVertex2f(drawAim.x-xfac,drawAim.y);
				glVertex2f(drawAim.x+xfac,drawAim.y);
				glVertex2f(drawAim.x,drawAim.y-0.03);
				glVertex2f(drawAim.x,drawAim.y+0.03);
			}
		glEnd();
		glLineWidth(1);
	glPopAttrib();
}

void Player::update(float dt)
{
	if(!classicAim) {

		if(keys[LEFT_ARROW])
			AimLeft(dt);
		if(keys[RIGHT_ARROW])
			AimRight(dt);
		if(keys[SPACE_BAR])
			addPower(dt);

	} else {

		if(keys['W'] || keys[UP_ARROW])
			aimUp(dt/2);
		if(keys['A'] || keys[LEFT_ARROW])
			aimLeft(dt/2);
		if(keys['S'] || keys[DOWN_ARROW])
			aimDown(dt/2);
		if(keys['D'] || keys[RIGHT_ARROW])
			aimRight(dt/2);
			
	}
}

int Player::GetKeyboardRef() const
{
	return keyboardRef;
}

bool Player::IsKeyboard(const int ref) const
{
	return (ref == keyboardRef);
}

void Player::SetCity(const int city, const vec2f pos, const float radius)
{
	cityNum = city;
	aimPoint = AIM_DIV / 4.0;
	power = POWER_MIN;
	cityPos = pos;
	cityRadius = radius;
	calcAim();
	ResetVAim();
}

int Player::GetCity() const
{
	return cityNum;
}

void Player::AimLeft(float dt)
{
	aimPoint += AIM_RATE * dt;
	if(highAim <= AIM_DIV && aimPoint > highAim)
		aimPoint = highAim;
	if(aimPoint > AIM_DIV)
		aimPoint -= AIM_DIV;
	calcAim();
}

void Player::AimRight(float dt)
{
	aimPoint -= AIM_RATE * dt;
	if(lowAim <= AIM_DIV && aimPoint < lowAim)
		aimPoint = lowAim;
	if(aimPoint < 0)
		aimPoint += AIM_DIV;
	calcAim();
}

void Player::SetAimPoint(const float aimP)
{
	aimPoint = aimP;
	calcAim();
}

float Player::GetAimPoint() const
{
	return aimPoint;
}

void Player::SetAimLimits(const float aimLow, const float aimHigh)
{
	lowAim = aimLow;
	highAim = aimHigh;
}

void Player::calcAim()
{
	float theta = 2.0 * aimPoint * pi / AIM_DIV;
	aim.x = cos(theta);
	aim.y = sin(theta);
}

vec2f Player::GetAim() const
{
	return aim;
}

void Player::addPower(float dt)
{
	power += dt;
	if(power > Max.y - Min.y)
		power = Max.y - Min.y;
}

void Player::dropPower(float dt)
{
	power -= dt;
	if(power < POWER_MIN)
		power = POWER_MIN;
}

float Player::Power() const
{
	return power;
}

void Player::SetPower(const float pwr)
{
	power = pwr;
}

void Player::ResetPower()
{
	power = POWER_MIN;
}

void Player::Fired()
{
	lastFireTime = clock();
}

bool Player::CanFire() const
{
	return ((clock() - lastFireTime) * 0.001 >= REFIRE_RATE);
}

void Player::aimLeft(float dt)
{
	vAim.x -= dt;
	if(vAim.x <= Min.x)
		vAim.x = Min.x;
	setAim();
}

void Player::aimRight(float dt)
{
	vAim.x += dt;
	if(vAim.x >= Max.x)
		vAim.x = Max.x;
	setAim();
}

void Player::aimUp(float dt)
{
	vAim.y += dt;
	if(vAim.y >= Max.y)
		vAim.y = Max.y;
	setAim();
}

void Player::aimDown(float dt)
{
	vAim.y -= dt;
	if(vAim.y <= Min.y)
		vAim.y = Min.y;
	setAim();
}

void Player::setAim()
{
	vec2f temp = vAim - vAimCentre;
	float rotation = atan2(temp.x,temp.y) * 180 / pi;
	aimPoint = (90.0 - rotation) * AIM_DIV / 360.0;
	calcAim();
	power = vec2f(vAim - cityPos).size();
}

void Player::ResetVAim()
{
	vAim.x = aim.x * cityRadius * 1.5;
	vAim.y = aim.y * cityRadius * 1.5;
	vAim += cityPos;
	vAim += aim * power;
}

void Player::UpdateCityPos(const vec2f pos)
{
	cityPos = pos;
}

void Player::CorrectVAim(bool earth)
{
	if(earth) {
		if(vAim.sizesq() <= 0.25) {
			vAim.normalize();
			vAim *= 0.5;
		}
	} else {
		if(vAim.y <= cityPos.y) {
			vAim.y = cityPos.y;
		}
	}
}

void Player::ToggleClassicAim()
{
	classicAim = !classicAim;
	ResetPower();
	ResetVAim();
}

bool Player::ClassicAim() const
{
	return classicAim;
}

void Player::SetVAimCentre(const vec2f centre)
{
	vAimCentre = centre;
}