#ifndef PLAYER_H
#define PLAYER_H

#define NUM_KEYS 256

#define AIM_RATE 100
#define AIM_DIV 512.0
#define POWER_MIN 0.06
#define REFIRE_RATE 1.0

#define NO_CITY -1
#include "vec.h"
extern const float pi;

void applyColour(int pn);

class Player {
	private:
		int keyboardRef;
		int playerNum;
		int cityNum;
		float aimPoint;
		float lowAim;
		float highAim;
		vec2f aim;
		float power;
		float lastFireTime;
		int missilesFired;
		bool classicAim;

		vec2f vAim;
		vec2f cityPos;
		vec2f vAimCentre;
		float cityRadius;

		void calcAim();
		void addPower(float dt);
		void dropPower(float dt);
		void aimLeft(float dt);
		void aimRight(float dt);
		void aimUp(float dt);
		void aimDown(float dt);
		void setAim();

	public:
		char keys[NUM_KEYS];
	
	public:
		Player(int ref, int num);
		void Draw() const;
		void AimDraw() const;
		void update(float dt);
		int GetKeyboardRef() const;
		bool IsKeyboard(const int ref) const;
		void SetCity(const int city, const vec2f pos, const float radius);
		int GetCity() const;
		void AimLeft(float dt);
		void AimRight(float dt);
		void SetAimPoint(const float aimP);
		float GetAimPoint() const;
		void SetAimLimits(const float aimLow, const float aimHigh);
		vec2f GetAim() const;
		float Power() const;
		void SetPower(const float pwr);
		void ResetPower();
		void Fired();
		bool CanFire() const;
		void ResetVAim();
		void UpdateCityPos(const vec2f pos);
		void CorrectVAim(bool earth);
		void ToggleClassicAim();
		bool ClassicAim() const;
		void SetVAimCentre(const vec2f centre);
};

#endif