#include "projectiles.h"

ParticleContainer::ParticleContainer(float red, float green, float blue, float spd){
	count = 0;
	r = red;
	g = green;
	b = blue;
	speed = spd;
	flag = false;
}

void ParticleContainer::drawPoints(){
	int i;
	glPushAttrib(GL_CURRENT_BIT);
	glBegin(GL_POINTS);
	for(i = 0; i < count; i++){
		glColor3f(r,g,b);
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
	}
	glEnd();
	glPopAttrib();
}

void ParticleContainer::updatePoints(float dt){
	int i;

	for(i = 0; i < count; i++){
		//change position
		points[i].Pos += points[i].Vel*points[i].speed*dt/10;
		//calculate life
		points[i].life -= dt;
		if(points[i].life < 0.0){
			//delete if life is too low
			deletePoint(i);
		}
	}
}

void ParticleContainer::createPoint(){
	Particle tempPart;
	if(count >= cMax)
		return;

	tempPart.Pos.x = 0;
	tempPart.Pos.y = 0;
	tempPart.Vel.x = (RAND_UNIT-0.5);
	tempPart.Vel.y = (RAND_UNIT-0.5);
	tempPart.Vel.normalize();
	if(speed == 0.0)
		tempPart.speed = RAND_UNIT;
	else
		tempPart.speed = speed;
	
	//gives a particle 2 seconds to live
	tempPart.life = 2;
	
	points.push_back(tempPart);
	count ++;
}

void ParticleContainer::createPoint(vec2f posSet, float heading){
	Particle tempPart;
	if(count >= cMax)
		return;

	tempPart.Pos.x = posSet.x;
	tempPart.Pos.y = posSet.y;
	if(heading >= 0){
		tempPart.Vel.x = cosf(heading);
		tempPart.Vel.y = sinf(heading);
	}
	else{
		tempPart.Vel.x = (RAND_UNIT-0.5);
		tempPart.Vel.y = (RAND_UNIT-0.5);
	}
	tempPart.Vel.normalize();
	if(speed == 0.0)
		tempPart.speed = RAND_UNIT;
	else
		tempPart.speed = speed;
	
	//gives a particle 2 seconds to live
	tempPart.life = 2;
	
	points.push_back(tempPart);
	count ++;
}

void ParticleContainer::createPoint(vec2f posSet, vec2f posAim){
	Particle tempPart;
	if(count >= cMax)
		return;

	tempPart.Pos.x = posSet.x;
	tempPart.Pos.y = posSet.y;
	tempPart.Vel.x = posAim.x - posSet.x;
	tempPart.Vel.y = posAim.y - posSet.y;
	tempPart.Vel.normalize();
	if(speed == 0.0)
		tempPart.speed = RAND_UNIT;
	else
		tempPart.speed = speed;

	//calculates life to die when at posAim
	float xdistance = posAim.x - posSet.x;
	float ydistance = posAim.y - posSet.y;
	float distance = sqrt(xdistance*xdistance + ydistance*ydistance);
	tempPart.life = distance/(tempPart.speed)*10;
	
	points.push_back(tempPart);
	count ++;
}

void ParticleContainer::deletePoint(int i){
	points.erase(points.begin()+i);
	count --;
}

void ParticleContainer::erase(){
	points.clear();
	count = 0;
}

Particle ParticleContainer::getNext(){
	Particle returnPart;
	if(!flag){
		it = points.begin();
		flag = TRUE;
	}
	while(!points.empty() && it != points.end()){
		returnPart = *it;
		it++;
		return returnPart;
	}
	flag = FALSE;
	returnPart.life = -1;
	return returnPart;
}

void ParticleContainer::changeColour(float red, float green, float blue){
	r = red;
	g = green;
	b = blue;
}

void ParticleContainer::ring(vec2f posSet){
	for(float i = 0; i < 2*PI; i += PI/16){
		createPoint(posSet, i);
	}
}

void ParticleContainer::burst(vec2f posSet){
	float x = speed;
	speed = 0;
	for(float i = 0; i < 2*PI; i += PI/16){
		createPoint(posSet);
	}
	speed = x;
}


void ParticleContainer::setSpeed(float spd)
{
	speed = spd;
}

int ParticleContainer::size()
{
	return (int) points.size();
}