#include "rawwin.h"
#include "glenable.h"
#include "control.h"

HWND hWnd;
LONG nWidth = WIN_SIZE_X;
LONG nHeight = WIN_SIZE_Y;
HDC hDC;
HGLRC hRC;

RAWINPUTDEVICE Rid;
vector<RI_STORE> InputCounter;
vector<string> Keyboards;

RI_STORE::RI_STORE(int ref)
{
	memset(keys,false,NUM_KEYS * sizeof(bool));
	memset(keyReleaseTime,0,NUM_KEYS * sizeof(float)); 
	memset(keyPushTime,0,NUM_KEYS * sizeof(float));
	keyboardRef = ref;
}

bool RI_STORE::IsPressed(USHORT key)
{
	if(key >= NUM_KEYS)
		return false;
	return ( keyReleaseTime[key] < keyPushTime[key] );
}

void RI_STORE::ReleaseKey(USHORT key)
{
	if(key >= NUM_KEYS)
		return;
	keys[key] = false;
	keyReleaseTime[key] = clock();
}

bool RI_STORE::PushKey(USHORT key)
{
	if(key >= NUM_KEYS)
		return false;
	if(!IsPressed(key)) {
		keys[key] = true;
		keyPushTime[key] = clock();
		return true;
	}
	return false;
}

void RegisterInput()
{
	Rid.usUsagePage = 0x01;
	Rid.usUsage = KEYBOARD;
	Rid.dwFlags = 0;
	if (RegisterRawInputDevices(&Rid, 1, sizeof(Rid)) == FALSE) printf("Error registering input\n");
}

bool InitWindow(string title)
{
	string szClassName = "WindowsApp";

	WNDCLASSEX WindowClass;
	WindowClass.cbSize = sizeof (WNDCLASSEX);
	WindowClass.style = WIN_CLASS_STYLE;
	WindowClass.lpfnWndProc = WindowProcedure;
	WindowClass.cbClsExtra = 0;
	WindowClass.cbWndExtra = 0;
	WindowClass.hInstance = GetModuleHandle(NULL);
	WindowClass.hIcon = LoadIcon (NULL, IDI_APPLICATION);
	WindowClass.hCursor = LoadCursor (NULL, IDC_ARROW);
	WindowClass.hbrBackground = (HBRUSH) COLOR_BACKGROUND;  
	WindowClass.lpszMenuName = NULL;
	WindowClass.lpszClassName = szClassName.c_str();
	WindowClass.hIconSm = LoadIcon (NULL, IDI_APPLICATION);

	if (!RegisterClassEx(&WindowClass)) return false;

	hWnd = CreateWindowEx( (DWORD) 0,
		(LPCTSTR) szClassName.c_str(),
		(LPCTSTR) title.c_str(),
		(DWORD) WIN_STYLE_FLAGS,
		(int) 0,//CW_USEDEFAULT,
		(int) 0,//CW_USEDEFAULT,
		(int) nWidth,  (int) nHeight,
		(HWND) HWND_DESKTOP,
		(HMENU) NULL,
		(HINSTANCE) GetModuleHandle(NULL),
		(LPVOID) NULL);
	ShowWindow(hWnd,SW_SHOW);
	return true;
}

LRESULT CALLBACK WindowProcedure(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INPUT:
			ProcessInput(lParam);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));
			break;
		default:
			return DefWindowProc (hwnd, message, wParam, lParam);
	}

	return 0;
}

void ProcessInput(LPARAM hRawInput)
{
	UINT cbSize;
	GetRawInputData((HRAWINPUT) hRawInput,
				(UINT) RID_INPUT,
				(LPVOID) NULL,
				(PUINT) &cbSize,
				(UINT) sizeof(RAWINPUTHEADER));

	LPBYTE lpbBuffer = new BYTE[cbSize];

	GetRawInputData((HRAWINPUT) hRawInput,
				(UINT) RID_INPUT,
				(LPVOID) lpbBuffer,
				(PUINT) &cbSize,
				(UINT) sizeof(RAWINPUTHEADER));

	RAWINPUT * pRawInput = (RAWINPUT *) lpbBuffer;
	
	
	if (pRawInput->header.dwType == RIM_TYPEKEYBOARD) {

		UINT size = BUF_SIZE;
		TCHAR tBuffer[BUF_SIZE];

		GetRawInputDeviceInfo(pRawInput->header.hDevice, RIDI_DEVICENAME, tBuffer, &size);
		//string temp = GetKeyboardName(tBuffer);
		int ref = KeyboardNum(tBuffer);

		if(ref == -1)
			ref = AddKeyboard(tBuffer);

		if(pRawInput->data.keyboard.Flags == KEY_UP || pRawInput->data.keyboard.Flags == KEY_UP_RIGHT) {

			InputCounter[ref].ReleaseKey(pRawInput->data.keyboard.VKey);
			//ConsoleUpdate(ref,true,pRawInput->data.keyboard.VKey,pRawInput->header.hDevice);
			keyUp(pRawInput->data.keyboard.VKey,ref);

		} else if(pRawInput->data.keyboard.Flags == KEY_DOWN || pRawInput->data.keyboard.Flags == KEY_DOWN_RIGHT) {

			if(InputCounter[ref].PushKey(pRawInput->data.keyboard.VKey)) {
				//ConsoleUpdate(ref,false,pRawInput->data.keyboard.VKey,pRawInput->header.hDevice);
				keyDown(pRawInput->data.keyboard.VKey,ref);
			}
		}
	} /*else if(pRawInput->header.dwType == RIM_TYPEMOUSE) {
		cout << pRawInput->data.mouse.usFlags << endl;
		cout << pRawInput->data.mouse.ulButtons << endl;
		cout << pRawInput->data.mouse.usButtonFlags << endl;
		cout << pRawInput->data.mouse.usButtonData << endl;
		cout << pRawInput->data.mouse.ulRawButtons << endl;
		cout << pRawInput->data.mouse.lLastX << endl;
		cout << pRawInput->data.mouse.lLastY << endl;
		cout << pRawInput->data.mouse.ulExtraInformation << endl;
	}*/
	free(lpbBuffer);
}

int AddKeyboard(const char * name)
{
	string temp(name);
	int i;
	for(i = 0; i < (int)Keyboards.size(); i++)
		if(Keyboards[i] == temp)
			return i;
	Keyboards.push_back(temp);
	RI_STORE RIS(i);
	InputCounter.push_back(RIS);
	return i;
}

int KeyboardNum(const char * name)
{
	string temp(name);
	for(int i = 0; i < (int)Keyboards.size(); i++)
		if(Keyboards[i] == temp)
			return i;
	return -1;
}
/*
void ConsoleUpdate(int keyboard, bool release, USHORT key, HANDLE hDev)
{
	string keyRef = GetKey(key);
	if(hDev != NULL)
		cout << "Handle " << hDev << " ";
	cout << "Keyboard " << keyboard << ": Key " << keyRef << " was ";
	if(release) cout << "released\n";
	else cout << "pressed\n";
}
*/
void EnableOpenGL()
{
	PIXELFORMATDESCRIPTOR pfd;
	int iFormat;

	// get the device context (DC)
	hDC = GetDC( hWnd );

	// set the pixel format for the DC
	memset( &pfd, 0, sizeof( pfd ) );
	pfd.nSize = sizeof( pfd );
	pfd.nVersion = 1;
	pfd.dwFlags = PIXEL_FLAGS;
	pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.cColorBits = 24;
	pfd.cDepthBits = 32;
	pfd.iLayerType = PFD_MAIN_PLANE;

	// picks the right format
	iFormat = ChoosePixelFormat( hDC, &pfd );
	// sets the format
	SetPixelFormat( hDC, iFormat, &pfd );

	// create and enable the render context (RC)
	hRC = wglCreateContext( hDC );
	// set to use our settings
	wglMakeCurrent( hDC, hRC );

	glClearDepth(5.0f);
	glEnable(GL_BLEND); 
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST); //used for anti-aliasing
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST); //used for anti-aliasing
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
	// glAlphaFunc(GL_GREATER,0.1f);
	// glEnable(GL_ALPHA_TEST);
	//glEnable(GL_DEPTH_TEST);
	//glDepthFunc(GL_LEQUAL);
	//glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glDepthMask(GL_TRUE);
}

void DisableOpenGL()
{
	wglMakeCurrent( NULL, NULL );
	wglDeleteContext( hRC );
	ReleaseDC( hWnd, hDC );
}

void DisplayLoop()
{
	MSG msg;
	float thisTime = 0;
	float lastTime = 0;
	lastTime = clock();
	while(true) {

		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			if(msg.message == WM_QUIT)
				break;
			else {
				TranslateMessage(&msg);	
				DispatchMessage(&msg);
			}
		} else {
			thisTime = clock();
			float dt = (thisTime - lastTime) * 0.001;
			lastTime = thisTime;
			update(dt);
			display();
			SwapBuffers(hDC);
		}
	}
}

string GetKeyboardName(char * buffer)
{
	string temp = buffer;
	string t2;

	if(!temp.compare(INBUILT))
		return "Root";

	int left = temp.find('&');
	if(left == (int) temp.npos)
		return "";
	int right = temp.find('&',left + 1);
	if(right == (int) temp.npos)
		return "";

	t2 = temp.substr(left,right-left);

	return t2;
}

bool GetActiveKeyboards(vector<int>& keyboards)
{
	UINT nDevices;
	PRAWINPUTDEVICELIST pRawInputDeviceList;
		// get how many devices
	if (GetRawInputDeviceList(NULL, &nDevices, sizeof(RAWINPUTDEVICELIST)) != 0)
		return false;
		// allocate spcae enough to get information about them
	if ((pRawInputDeviceList = (PRAWINPUTDEVICELIST) malloc(sizeof(RAWINPUTDEVICELIST) * nDevices)) == NULL)
		return false;
		// get information
	if (GetRawInputDeviceList(pRawInputDeviceList, &nDevices, sizeof(RAWINPUTDEVICELIST)) == (UINT)(-1))
		return false;

	for(int i = 0; i < (int)nDevices; i ++) {
		UINT size = 256;
		TCHAR tBuffer[256] = {0};
		
		RID_DEVICE_INFO rdi;
		rdi.cbSize = sizeof(RID_DEVICE_INFO);
		UINT cbSize = rdi.cbSize;

			// get device name
		if(GetRawInputDeviceInfo(pRawInputDeviceList[i].hDevice, RIDI_DEVICENAME, tBuffer, &size) < 0)
			continue;

			// get device info
		if(GetRawInputDeviceInfo(pRawInputDeviceList[i].hDevice, RIDI_DEVICEINFO, &rdi, &cbSize) < 0)
			continue;

			// if its a keyboard
		if(rdi.dwType == RIM_TYPEKEYBOARD) {
			//string temp = GetKeyboardName(tBuffer);
			int ref = KeyboardNum(tBuffer);
				// if its been/being used, add it to the list
			if(ref != -1)
				keyboards.push_back(ref);
		}
	}
	free(pRawInputDeviceList);
	return true;
}