#ifndef RAW_WINDOWS_H
#define RAW_WINDOWS_H

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0502
#endif

#include <windows.h>
#include <winuser.h>
#include <wingdi.h>
#include <windowsx.h>

#include <iostream>
#include <string>
#include <sstream>
#include <vector>

using namespace std;

#ifndef WIN_SIZE_X
#define WIN_SIZE_X 1280
#endif

#ifndef WIN_SIZE_Y
#define WIN_SIZE_Y 768
#endif

#define BUF_SIZE 256
#define WIN_STYLE_FLAGS (WS_MINIMIZEBOX | WS_SYSMENU | GL_STYLE_FLAGS)
#define WIN_CLASS_STYLE (CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_OWNDC)
#define INBUILT "\\\\?\\Root#"

#define MOUSE 0x02
#define KEYBOARD 0x06

#define NUM_KEYS 256

#define KEY_DOWN (RI_KEY_MAKE)
#define KEY_UP (RI_KEY_BREAK)
#define KEY_DOWN_RIGHT (2)
#define KEY_UP_RIGHT (3)

#define LEFT_ARROW 37
#define UP_ARROW 38
#define RIGHT_ARROW 39
#define DOWN_ARROW 40

#define BACKSPACE 8
#define TAB 9
#define ENTER 13
#define SHIFT 16
#define CONTROL 17
#define ALT 18
#define CAPS_LOCK 20
#define ESCAPE 27
#define SPACE_BAR 32

#define PAGE_UP 33
#define PAGE_DOWN 34
#define END 35
#define HOME 36
#define INSERT 45
#define DELETE_KEY 46

#define CASE_DIFF 32
#define A_CHAR 65
#define Z_CHAR 90

#define F1_DIFF 111

#define NORM_NUM_0 48
#define NORM_NUM_9 57
#define NUMPAD_0 96
#define NUMPAD_9 105
#define NUMBER_DIFF (NORM_NUM_0)
#define NUMPAD_DIFF (NUMPAD_0 - NORM_NUM_0)

struct RI_STORE {
	bool keys[NUM_KEYS];
	float keyPushTime[NUM_KEYS];
	float keyReleaseTime[NUM_KEYS];
	int keyboardRef;

	RI_STORE(int Ref);
	bool IsPressed(USHORT key);	
	void ReleaseKey(USHORT key);
	bool PushKey(USHORT key);		// push it if not already pushed
};


void RegisterInput();
bool InitWindow(string title);
LRESULT CALLBACK WindowProcedure(HWND, UINT, WPARAM, LPARAM);
void ProcessInput(LPARAM);
int AddKeyboard(const char *);
int KeyboardNum(const char *);
//void ConsoleUpdate(int, bool, USHORT, HANDLE hDev= NULL);
string GetKeyboardName(char * buffer);
bool GetActiveKeyboards(vector<int>& keyboards);

void EnableOpenGL();
void DisableOpenGL();
void DisplayLoop();

#endif