#include "rockets.h"

Rockets::Rockets(float red, float green, float blue, float spd, Explosions * expld){
	count = 0;
	r = red;
	g = green;
	b = blue;
	speed = spd;
	explosions = expld;
}

Rockets::Rockets(float spd, Explosions * expld){
	speed = spd;
	explosions = expld;
	count = 0;
}

void Rockets::drawPoints(){
	int i;
	glPushAttrib(GL_CURRENT_BIT);
	glLineWidth(3);
	glBegin(GL_LINES);
	for(i = 0; i < count; i++){
		glColor3f(r,g,b);
		glVertex2f(points[i].Pos.x, points[i].Pos.y);
		glVertex2f(points[i].Pos.x - points[i].Vel.x/50,points[i].Pos.y - points[i].Vel.y/50);
	}
	glEnd();
	glLineWidth(1);
	glPopAttrib();
	trail.drawPoints();
}

void Rockets::updatePoints(float dt){
	int i;
	timer += dt;
	for(i = 0; i < (int) points.size(); i++){
		//change position
		points[i].Pos += points[i].Vel*points[i].speed*dt/10;
		//calculate life
		points[i].life -= dt;
		vec2f tempVec;
		tempVec.x = points[i].Pos.x - points[i].Vel.x/50;
		tempVec.y = points[i].Pos.y - points[i].Vel.y/50;
		if(timer > 0.05){
			trail.createPoint(tempVec, points[i].Vel);
		}
		if(points[i].Pos.x < Min.x || points[i].Pos.x > Max.x){
			deletePoint(i);
			continue;
		}
		if(points[i].life < 0.0){
			deletePoint(i);
		}
	}
	trail.updatePoints(dt);
	if(timer > 0.05){
		timer = 0;
	}
}

void Rockets::createPoint(vec2f posSet, float heading){
	Particle tempPart;
	if(count >= cMax)
		return;
	
	tempPart.Pos.x = posSet.x;
	tempPart.Pos.y = posSet.y;
	if(heading >= 0){
		tempPart.Vel.x = cosf(heading);
		tempPart.Vel.y = sinf(heading);
	}
	else{
		tempPart.Vel.x = (RAND_UNIT-0.5);
		tempPart.Vel.y = (RAND_UNIT-0.5);
	}
	tempPart.Vel.normalize();
	if(speed == 0.0)
		tempPart.speed = RAND_UNIT/2+0.5;
	else
		tempPart.speed = speed;
	
	//gives a particle 20 seconds to live
	tempPart.life = 20;
	
	points.push_back(tempPart);
	count ++;
}

void Rockets::createPoint(vec2f posSet, vec2f posAim){
	Particle tempPart;
	if(count >= cMax)
		return;
	
	tempPart.Pos.x = posSet.x;
	tempPart.Pos.y = posSet.y;
	tempPart.Vel.x = posAim.x - posSet.x;
	tempPart.Vel.y = posAim.y - posSet.y;
	tempPart.Vel.normalize();
	if(speed == 0.0)
		tempPart.speed = RAND_UNIT/2+0.5;
	else
		tempPart.speed = speed;

	//calculates life to die when at posAim
	float xdistance = posAim.x - posSet.x;
	float ydistance = posAim.y - posSet.y;
	float distance = sqrt(xdistance*xdistance + ydistance*ydistance);
	tempPart.life = distance/(tempPart.speed)*10;
	
	points.push_back(tempPart);
	count ++;
}

void Rockets::deletePoint(int i)
{
	if(points[i].Pos.y > Max.y - Max.y * 0.25)
		return;
	if(points[i].Pos.x > Min.x && points[i].Pos.x < Max.x && points[i].Pos.y < Max.y){
		if(explosions)
			explosions -> createPoint(points[i].Pos);
	}
	ParticleContainer::deletePoint(i);
}